# GUID Used to further identify office documents
OLE_CLSID_GUIDs = {
    # GUID v0 (0)
    "00020803-0000-0000-C000-000000000046": "document/office/word",  # "MS Graph Chart"
    "00020900-0000-0000-C000-000000000046": "document/office/word",  # "MS Word95"
    "00020901-0000-0000-C000-000000000046": "document/office/word",  # "MS Word 6.0 - 7.0 Picture"
    "00020906-0000-0000-C000-000000000046": "document/office/word",  # "MS Word97"
    "00020907-0000-0000-C000-000000000046": "document/office/word",  # "MS Word"
    "00020C01-0000-0000-C000-000000000046": "document/office/excel",  # "Excel"
    "00020821-0000-0000-C000-000000000046": "document/office/excel",  # "Excel"
    "00020820-0000-0000-C000-000000000046": "document/office/excel",  # "Excel97"
    "00020810-0000-0000-C000-000000000046": "document/office/excel",  # "Excel95"
    "00021a14-0000-0000-C000-000000000046": "document/office/visio",  # "Visio"
    "0002CE02-0000-0000-C000-000000000046": "document/office/equation",  # "MS Equation 3.0"
    "0003000A-0000-0000-C000-000000000046": "document/office/paintbrush",  # "Paintbrush Picture",
    "0003000C-0000-0000-C000-000000000046": "document/office/package",  # "Package"
    "000C1084-0000-0000-C000-000000000046": "document/installer/windows",  # "Installer Package (MSI)"
    "00020D0B-0000-0000-C000-000000000046": "document/office/email",  # "MailMessage"
    # GUID v1 (Timestamp & MAC-48)
    "29130400-2EED-1069-BF5D-00DD011186B7": "document/office/wordpro",  # "Lotus WordPro"
    "46E31370-3F7A-11CE-BED6-00AA00611080": "document/office/word",  # "MS Forms 2.0 MultiPage"
    "5512D110-5CC6-11CF-8D67-00AA00BDCE1D": "document/office/word",  # "MS Forms 2.0 HTML SUBMIT"
    "5512D11A-5CC6-11CF-8D67-00AA00BDCE1D": "document/office/word",  # "MS Forms 2.0 HTML TEXT"
    "5512D11C-5CC6-11CF-8D67-00AA00BDCE1D": "document/office/word",  # "MS Forms 2.0 HTML Hidden"
    "64818D10-4F9B-11CF-86EA-00AA00B929E8": "document/office/powerpoint",  # "MS PowerPoint Presentation"
    "64818D11-4F9B-11CF-86EA-00AA00B929E8": "document/office/powerpoint",  # "MS PowerPoint Presentation"
    "11943940-36DE-11CF-953E-00C0A84029E9": "document/office/word",  # "MS Photo Editor 3.0 Photo"
    "B801CA65-A1FC-11D0-85AD-444553540000": "document/pdf",  # "Adobe Acrobat Document"
    "A25250C4-50C1-11D3-8EA3-0090271BECDD": "document/office/wordperfect",  # "WordPerfect Office"
    "C62A69F0-16DC-11CE-9E98-00AA00574A4F": "document/office/word",  # "Microsoft Forms 2.0 Form"
    "F4754C9B-64F5-4B40-8AF4-679732AC0607": "document/office/word",  # Word.Document.12
    "BDD1F04B-858B-11D1-B16A-00C0F0283628": "document/office/word",  # Doc (see CVE2012-0158)
}

# Assemblyline type to file extension mapping
type_to_extension = {
    "archive/chm": ".chm",
    "archive/iso": ".iso",
    "archive/rar": ".rar",
    "archive/udf": ".udf",
    "archive/vhd": ".vhd",
    "archive/zip": ".zip",
    "archive/7-zip": ".7z",
    "audiovisual/flash": ".swf",
    "code/a3x": ".a3x",
    "code/batch": ".bat",
    "code/c": ".c",
    "code/csharp": ".cs",
    "code/hta": ".hta",
    "code/html": ".html",
    "code/java": ".java",
    "code/javascript": ".js",
    "code/jscript": ".js",
    "code/pdfjs": ".js",
    "code/perl": ".pl",
    "code/php": ".php",
    "code/ps1": ".ps1",
    "code/python": ".py",
    "code/ruby": ".rb",
    "code/shell": ".sh",
    "code/vbe": ".vbe",
    "code/vbs": ".vbs",
    "code/wsf": ".wsf",
    "document/installer/windows": ".msi",
    "document/office/excel": ".xls",
    "document/office/mhtml": ".mhtml",
    "document/office/ole": ".doc",
    "document/office/powerpoint": ".ppt",
    "document/office/rtf": ".doc",
    "document/office/unknown": ".doc",
    "document/office/visio": ".vsd",
    "document/office/word": ".doc",
    "document/office/wordperfect": "wp",
    "document/office/wordpro": "lwp",
    "document/office/onenote": ".one",
    "document/pdf": ".pdf",
    "document/email": ".eml",
    "executable/web/wasm": ".wasm",
    "executable/windows/pe32": ".exe",
    "executable/windows/pe64": ".exe",
    "executable/windows/dll32": ".dll",
    "executable/windows/dll64": ".dll",
    "executable/windows/dos": ".exe",
    "executable/windows/com": ".exe",
    "executable/linux/elf32": ".elf",
    "executable/linux/elf64": ".elf",
    "executable/linux/so32": ".so",
    "executable/linux/so64": ".so",
    "java/jar": ".jar",
    "silverlight/xap": ".xap",
    "shortcut/windows": ".lnk",
    "text/windows/registry": ".reg",
}

# Regex patterns used to find Assemblyline type in the reported magic labels
# Magic bytes translated to possible libmagic labels: https://en.wikipedia.org/wiki/List_of_file_signatures
magic_patterns = [
    {"al_type": "network/tnef", "regex": r"Transport Neutral Encapsulation Format"},
    {"al_type": "archive/chm", "regex": r"MS Windows HtmlHelp Data"},
    {"al_type": "executable/web/wasm", "regex": r"WebAssembly \(wasm\) binary module"},
    {"al_type": "executable/windows/dll64", "regex": r"pe32\+[^\|]+dll[^\|]+x86\-64"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L159
    {"al_type": "executable/windows/dll64", "regex": r"pe32\+[^\|]+dll[^\|]+windows"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L157
    {"al_type": "executable/windows/dll32", "regex": r"pe32[^\|]+dll"},
    {"al_type": "executable/windows/pe64", "regex": r"pe32\+[^\|]+x86\-64[^\|]+windows"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L155
    {"al_type": "executable/windows/pe64", "regex": r"pe32\+[^\|]+windows"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L153
    {"al_type": "executable/windows/pe32", "regex": r"pe32[^\|]+windows"},
    {"al_type": "executable/windows/ia/dll64", "regex": r"pe32\+?[^\|]+dll[^\|]+Intel Itanium[^\|]+windows"},
    {"al_type": "executable/windows/ia/pe64", "regex": r"pe32\+?[^\|]+Intel Itanium[^\|]+windows"},
    {"al_type": "executable/windows/arm/dll64", "regex": r"pe32\+?[^\|]+dll[^\|]+Aarch64[^\|]+windows"},
    {"al_type": "executable/windows/arm/pe64", "regex": r"pe32\+?[^\|]+Aarch64[^\|]+windows"},
    {"al_type": "executable/windows/pe", "regex": r"pe unknown[^\|]+windows"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L183
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L185
    {"al_type": "executable/windows/dos", "regex": r"(ms-)?dos executable"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L187
    {"al_type": "executable/windows/com", "regex": r"^com executable"},
    {"al_type": "executable/windows/dos", "regex": r"^8086 relocatable"},
    {"al_type": "executable/windows/coff", "regex": r"^MS Windows COFF"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_elf.yara
    {"al_type": "executable/linux/elf32", "regex": r"^elf 32-bit (l|m)sb executable"},
    {"al_type": "executable/linux/elf64", "regex": r"^elf 64-bit (l|m)sb executable"},
    {"al_type": "executable/linux/pie32", "regex": r"^elf 32-bit (l|m)sb pie executable"},
    {"al_type": "executable/linux/pie64", "regex": r"^elf 64-bit (l|m)sb pie executable"},
    {"al_type": "executable/linux/so32", "regex": r"^elf 32-bit (l|m)sb +shared object"},
    {"al_type": "executable/linux/so64", "regex": r"^elf 64-bit (l|m)sb +shared object"},
    {"al_type": "executable/linux/coff32", "regex": r"^(Intel 80386|i386|80386) COFF"},
    {"al_type": "executable/linux/coff64", "regex": r"^64-bit XCOFF"},
    {"al_type": "executable/linux/ia/coff64", "regex": r"^Intel ia64 COFF"},
    {"al_type": "executable/linux/misp/ecoff", "regex": r"^MIPS[^\|]+ ECOFF"},
    {"al_type": "executable/linux/a.out", "regex": r"^a.out"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_macho.yara
    {"al_type": "executable/mach-o", "regex": r"^Mach-O"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L171
    {"al_type": "archive/7-zip", "regex": r"^7-zip archive data"},
    {"al_type": "archive/ace", "regex": r"^ACE archive data"},
    {"al_type": "archive/asar", "regex": r"^Electron ASAR archive"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L173
    {"al_type": "archive/bzip2", "regex": r"^bzip2 compressed data"},
    {"al_type": "archive/cabinet", "regex": r"^installshield cab"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_cab.yara
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L199
    {"al_type": "archive/cabinet", "regex": r"^microsoft cabinet archive data"},
    {"al_type": "archive/cpio", "regex": r"cpio archive"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L175
    {"al_type": "archive/gzip", "regex": r"^gzip compressed data"},
    {"al_type": "archive/iso", "regex": r"ISO 9660"},
    {"al_type": "archive/lzma", "regex": r"^LZMA compressed data"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_rar.yara
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L179
    {"al_type": "archive/rar", "regex": r"^rar archive data"},
    {"al_type": "archive/squashfs", "regex": r"^Squashfs filesystem"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_tar.yara
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L177
    {"al_type": "archive/tar", "regex": r"^(GNU|POSIX) tar archive"},
    {"al_type": "archive/ar", "regex": r"^current ar archive"},
    {"al_type": "archive/vhd", "regex": r"^Microsoft Disk Image"},
    {"al_type": "archive/vmdk", "regex": r"^VMware4? disk image"},
    {"al_type": "archive/xz", "regex": r"^XZ compressed data"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_zip.yara
    {"al_type": "archive/zip", "regex": r"^zip archive data"},
    {"al_type": "archive/zstd", "regex": r"^Zstandard compressed data"},
    {"al_type": "archive/zpaq", "regex": r"^ZPAQ file"},
    {"al_type": "network/tcpdump", "regex": r"^(tcpdump|pcap)"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_pdf.yara
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L149
    {"al_type": "document/pdf", "regex": r"^pdf document"},
    {"al_type": "document/epub", "regex": r"^EPUB document"},
    {"al_type": "document/mobi", "regex": r"^Mobipocket E-book"},
    {"al_type": "resource/map/warcraft3", "regex": r"^Warcraft III map file$"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L169
    {"al_type": "image/bmp", "regex": r"^pc bitmap"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L145
    {"al_type": "image/gif", "regex": r"^gif image data"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L147
    {"al_type": "image/jpg", "regex": r"^jpeg image data"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L151
    {"al_type": "image/png", "regex": r"^png image data"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L167
    {"al_type": "image/tiff", "regex": r"^TIFF image data"},
    {"al_type": "image/webp", "regex": r"Web/P image"},
    {"al_type": "document/installer/windows", "regex": r"(Installation Database|Windows Installer)"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L141
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L143
    {"al_type": "document/office/excel", "regex": r"Microsoft[^\|]+Excel"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L135
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L137
    {"al_type": "document/office/powerpoint", "regex": r"Microsoft.*PowerPoint"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L131
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L133
    {"al_type": "document/office/word", "regex": r"Microsoft[^\|]+Word"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_rtf.yara
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L139
    {"al_type": "document/office/rtf", "regex": r"Rich Text Format"},
    {"al_type": "document/office/ole", "regex": r"OLE 2"},
    {"al_type": "document/office/hwp", "regex": r"Hangul \(Korean\) Word Processor File"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_ole_cf.yara
    {"al_type": "document/office/unknown", "regex": r"Composite Document File|CDFV2"},
    {"al_type": "document/office/unknown", "regex": r"Microsoft[^\|]+(OOXML|Document)"},
    {"al_type": "document/office/unknown", "regex": r"Number of (Characters|Pages|Words)"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_swf.yara
    {"al_type": "audiovisual/flash", "regex": r"Macromedia Flash"},
    {"al_type": "code/autorun", "regex": r"microsoft windows autorun"},
    {"al_type": "code/batch", "regex": r"dos batch file"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L181
    {"al_type": "java/jar", "regex": r"[ (]Jar[) ]"},
    # Supported by https://github.com/EmersonElectricCo/fsf/blob/15303aa298414397f9aa5d19ca343040a0fe0bbd/fsf-server/yara/ft_java_class.yara
    {"al_type": "java/class", "regex": r"java class data"},
    {"al_type": "resource/pyc", "regex": r"python [^\|]+byte"},
    {"al_type": "resource/pyc", "regex": r"^Byte-compiled Python module"},
    {"al_type": "resource/yara/v3", "regex": r"^YARA 3.x compiled rule set"},
    {"al_type": "android/apk", "regex": r"Android package \(APK\)"},
    {"al_type": "code/xml", "regex": r"OpenGIS KML"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L161
    {"al_type": "code/xml", "regex": r"xml"},
    {"al_type": "image/tim", "regex": r"TIM image"},
    {"al_type": "network/sff", "regex": r"Frame Format"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L197
    {"al_type": "shortcut/windows", "regex": r"^MS Windows shortcut"},
    {"al_type": "document/email", "regex": r"Mime entity text"},
    {"al_type": "document/email", "regex": r"MIME entity, ASCII text"},
    {"al_type": "document/email", "regex": r"^message/rfc822"},
    {"al_type": "metadata/sysmon/evt", "regex": r"MS Windows Vista(-8.1)? Event Log"},
    {"al_type": "metadata/sysmon/evt", "regex": r"MS Windows 10-11 Event Log"},
    {"al_type": "metadata/minidump", "regex": r"Mini DuMP crash report"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L165
    {"al_type": "image/emf", "regex": r"Windows Enhanced Metafile"},
    {"al_type": "resource/msvc", "regex": r"MSVC \.res"},
    {"al_type": "pgp/pubkey", "regex": r"^PGP public key"},
    {"al_type": "pgp/privkey", "regex": r"^PGP private key block"},
    {"al_type": "pgp/encrypted", "regex": r"^PGP RSA encrypted session key"},
    {"al_type": "pgp/message", "regex": r"^PGP message Public-Key Encrypted Session Key"},
    {"al_type": "pgp/symmetric", "regex": r"^PGP message Symmetric-Key Encrypted Session Key"},
    {"al_type": "gpg/symmetric", "regex": r"^GPG symmetrically encrypted data"},
    {"al_type": "video/asf", "regex": r"^Microsoft ASF"},
    # Supported by https://github.com/mitre/multiscanner/blob/86e0145ba3c4a34611f257dc78cd2482ed6358db/multiscanner/modules/Metadata/fileextensions.py#L201
    {"al_type": "code/php", "regex": r"^PHP script"},
]

# LibMagic mimetypes that we blindly trust to assign an Assemblyline type
trusted_mimes = {
    # Mpeg Audio
    "audio/mp2": "audio/mp2",
    "audio/x-mp2": "audio/mp2",
    "audio/mpeg": "audio/mp3",
    "audio/mp3": "audio/mp3",
    "audio/mpg": "audio/mp3",
    "audio/x-mpeg": "audio/mp3",
    "audio/x-mp3": "audio/mp3",
    "audio/x-mpg": "audio/mp3",
    "audio/x-mp4a-latm": "audio/mp4",
    "audio/x-m4a": "audio/mp4",
    "audio/m4a": "audio/mp4",
    # Wav Audio
    "audio/x-wav": "audio/wav",
    "audio/wav": "audio/wav",
    "audio/vnd.wav": "audio/wav",
    # Ogg Audio
    "audio/ogg": "audio/ogg",
    "audio/x-ogg": "audio/ogg",
    # S3M Audio
    "audio/s3m": "audio/s3m",
    "audio/x-s3m": "audio/s3m",
    # MIDI Audio
    "audio/midi": "audio/midi",
    "audio/x-midi": "audio/midi",
    # Mpeg video
    "video/mp4": "video/mp4",
    # Avi video
    "video/x-msvideo": "video/avi",
    "video/x-avi": "video/avi",
    "video/avi": "video/avi",
    "video/msvideo": "video/avi",
    # Divx video
    "video/divx": "video/divx",
    "video/vnd.divx": "video/divx",
    # Quicktime video
    "video/quicktime": "video/quicktime",
    # ASF video
    "video/x-ms-asf": "video/asf",
    # Source code C/C++
    "text/x-c++": "text/plain",
    "text/x-c": "text/plain",
    # Configuration file
    "application/x-wine-extension-ini": "text/ini",
    # Python
    "text/x-python": "code/python",
    "text/x-script.python": "code/python",
    "application/x-bytecode.python": "resource/pyc",
    "text/x-bytecode.python": "resource/pyc",
    # PHP
    "text/x-php": "code/php",
    # XML file
    "text/xml": "code/xml",
    # SGML file
    "text/sgml": "code/sgml",
    # HTML file
    "text/html": "text/plain",
    # Shell script
    "text/x-shellscript": "code/shell",
    # RTF
    "text/rtf": "document/office/rtf",
    # Troff
    "text/troff": "text/troff",
    # Java
    # The text/x-java mime type is not a trusted mime to map to code/java as there are false positives with this.
    # But it is good enough to confirm that the type is at least text/plain.
    # A type of text/plain will then get sent to the yara identification stage.
    "text/x-java": "text/plain",
    # Batch
    "text/x-msdos-batch": "code/batch",
    # Registry file
    "text/x-ms-regedit": "text/windows/registry",
    # Sysmon EVTX file
    "application/x-ms-evtx": "metadata/sysmon/evt",
    # JSON file
    "application/json": "text/json",
    # Autorun files
    "application/x-setupscript": "code/autorun",
    # Bittorrent files
    "application/x-bittorrent": "application/torrent",
    "application/x-torrent": "application/torrent",
    # Database files
    "application/x-dbf": "db/dbf",
    "application/x-sqlite3": "db/sqlite",
    # Font
    "application/vnd.ms-opentype": "resource/font/opentype",
    "application/x-font-sfn": "resource/font/x11",
    # Image Icon
    "image/vnd.microsoft.icon": "image/icon",
    "application/ico": "image/icon",
    "image/ico": "image/icon",
    "image/icon": "image/icon",
    "image/x-ico": "image/icon",
    "image/x-icon": "image/icon",
    "text/ico": "image/icon",
    "image/x-icns": "image/icon",
    # Image gif
    "image/gif": "image/gif",
    # Image WebP
    "image/webp": "image/webp",
    # Image BMP
    "image/bmp": "image/bmp",
    "image/x-bmp": "image/bmp",
    "image/x-ms-bmp": "image/bmp",
    # Image metafile
    "image/wmf": "image/wmf",
    # Image SVG
    "image/svg": "image/svg",
    "image/svg+xml": "image/svg",
    # Image JPEG
    "image/jpeg": "image/jpg",
    "image/pjpeg": "image/jpg",
    # Image PNG
    "image/png": "image/png",
    # Image TGA
    "image/x-tga": "image/tga",
    "image/x-icb": "image/tga",
    # Image TIFF
    "image/tiff": "image/tiff",
    # Image Cursor
    "image/x-win-bitmap": "image/cursor",
    # Office Outlook email
    "application/vnd.ms-outlook": "document/office/email",
    # Office Powerpoint
    "application/vnd.openxmlformats-officedocument.presentationml.presentation": "document/office/powerpoint",
    "application/vnd.ms-powerpoint": "document/office/powerpoint",
    # Office Excel
    "application/vnd.ms-excel": "document/office/excel",
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet": "document/office/excel",
    # Office Word
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document": "document/office/word",
    "application/msword": "document/office/word",
    # Office encrypted docs
    "application/encrypted": "document/office/passwordprotected",
    # MSI file
    "application/vnd.ms-msi": "document/installer/windows",
    "application/x-msi": "document/installer/windows",
    # PDF Document
    "application/pdf": "document/pdf",
    # Postscript document
    "application/postscript": "document/ps",
    # Open Document files
    "application/vnd.oasis.opendocument.chart": "document/odt/chart",
    "application/vnd.oasis.opendocument.chart-template": "document/odt/chart",
    "application/vnd.oasis.opendocument.database": "db/odt",
    "application/vnd.oasis.opendocument.formula": "document/odt/formula",
    "application/vnd.oasis.opendocument.formula-template": "document/odt/formula",
    "application/vnd.oasis.opendocument.graphics": "document/odt/graphics",
    "application/vnd.oasis.opendocument.graphics-flat-xml": "document/odt/graphics",
    "application/vnd.oasis.opendocument.graphics-template": "document/odt/graphics",
    "application/vnd.oasis.opendocument.presentation": "document/odt/presentation",
    "application/vnd.oasis.opendocument.presentation-flat-xml": "document/odt/presentation",
    "application/vnd.oasis.opendocument.presentation-template": "document/odt/presentation",
    "application/vnd.oasis.opendocument.spreadsheet": "document/odt/spreadsheet",
    "application/vnd.oasis.opendocument.spreadsheet-flat-xml": "document/odt/spreadsheet",
    "application/vnd.oasis.opendocument.spreadsheet-template": "document/odt/spreadsheet",
    "application/vnd.oasis.opendocument.text": "document/odt/text",
    "application/vnd.oasis.opendocument.text-flat-xml": "document/odt/text",
    "application/vnd.oasis.opendocument.text-template": "document/odt/text",
    "application/vnd.oasis.opendocument.text-master": "document/odt/text",
    "application/vnd.oasis.opendocument.text-master-template": "document/odt/text",
    "application/vnd.oasis.opendocument.web": "document/odt/web",
    # Archives
    "application/x-7z-compressed": "archive/7-zip",
    "application/x-tar": "archive/tar",
    "application/x-tarapplication/x-dbt": "archive/tar",
    "application/gzip": "archive/gzip",
    "application/vnd.ms-tnef": "archive/tnef",
    "application/x-cpio": "archive/cpio",
    "application/x-archive": "archive/ar",
    "application/zip": "archive/zip",
    "application/zlib": "archive/zlib",
    "application/x-arj": "archive/arj",
    "application/x-lzip": "archive/lzip",
    "application/x-lzh-compressed": "archive/lzh",
    "application/x-ms-compress-szdd": "archive/szdd",
    "application/x-arc": "archive/arc",
    "application/x-iso9660-image": "archive/iso",
    "application/x-rar": "archive/rar",
    "application/x-virtualbox-vhd": "archive/vhd",
    "application/x-xz": "archive/xz",
    "application/vnd.ms-cab-compressed": "archive/cabinet",
    "application/zstd": "archive/zstd",
    "application/x-zstd": "archive/zstd",
    "application/x-bzip2": "archive/bzip2",
    "application/java-archive": "java/jar",
    # JAVA Class
    "application/x-java-applet": "java/class",
    # EPUB
    "application/epub+zip": "document/epub",
    # Packet capture
    "application/vnd.tcpdump.pcap": "network/tcpdump",
    "message/rfc822": "document/email",
    "text/calendar": "text/calendar",
    "application/x-mach-binary": "executable/mach-o",
    "application/x-gettext-translation": "resource/mo",
    "application/x-hwp": "document/office/hwp",
    "application/vnd.iccprofile": "metadata/iccprofile",
    "application/vnd.lotus-1-2-3": "document/lotus/spreadsheet",
    # Firefox modules
    "application/x-xpinstall": "application/mozilla/extension",
    # Chrome extensions
    "application/x-chrome-extension": "application/chrome/extension",
    # Android
    "application/vnd.android.package-archive": "android/apk",
}

# LibMagic mimetypes that we will fallback to when we can't determine a type
untrusted_mimes = {
    "application/javascript": "code/javascript",
    "application/x-powershell": "code/ps1",
    "text/x-java": "code/java",
    "text/html": "code/html",
    "text/x-c++": "code/c++",
    "text/x-c": "code/c",
}



from collections.abc import Mapping
from typing import AnyStr, Dict, List, Optional, Union
from typing import Mapping as _Mapping

from assemblyline.common.uid import get_id_from_data


def strip_nulls(d):
    if isinstance(d, dict):
        new_dict = {}
        for k, v in list(d.items()):
            v = strip_nulls(v) if v is not None else None
            # Assess if stripped value is null, if not then add it to the new dictionary returned
            # allow empty value
            if v is not None:
                new_dict[k] = v
        return new_dict
    elif isinstance(d, list):
        return [strip_nulls(v) for v in d if v is not None]
    else:
        return d


def recursive_update(d: Dict, u: _Mapping,
                     stop_keys: List[AnyStr] = [],
                     allow_recursion: bool = True) -> Union[Dict, _Mapping]:
    if d is None:
        return u

    if u is None:
        return d

    for k, v in u.items():
        if isinstance(v, Mapping) and allow_recursion:
            d[k] = recursive_update(d.get(k, {}), v, stop_keys=stop_keys, allow_recursion=k not in stop_keys)
        else:
            d[k] = v

    return d


def get_recursive_delta(d1: Union[Dict, Mapping], d2: Union[Dict, Mapping],
                        stop_keys: List[AnyStr] = [],
                        allow_recursion: bool = True) -> Dict:
    if d1 is None:
        return d2

    if d2 is None:
        return d1

    out = {}
    for k1, v1 in d1.items():
        if isinstance(v1, Mapping) and allow_recursion:
            internal = get_recursive_delta(v1, d2.get(k1, {}), stop_keys=stop_keys, allow_recursion=k1 not in stop_keys)
            if internal:
                out[k1] = internal
        else:
            if k1 in d2:
                v2 = d2[k1]
                if v1 != v2:
                    out[k1] = v2

    for k2, v2 in d2.items():
        if k2 not in d1:
            out[k2] = v2

    return out


def get_recursive_sorted_tuples(data: Dict):
    def sort_lists(ldata: List):
        new_list = []
        for i in ldata:
            if isinstance(i, list):
                i = sort_lists(i)
            elif isinstance(i, dict):
                i = get_recursive_sorted_tuples(i)

            new_list.append(i)
        return new_list

    items = []
    for k, v in sorted(data.items()):
        if isinstance(v, dict):
            v = get_recursive_sorted_tuples(v)
        elif isinstance(v, list):
            v = sort_lists(v)

        items.append((k, v))

    return items


def get_dict_fingerprint_hash(data: Dict):
    return get_id_from_data(str(get_recursive_sorted_tuples(data)))


def flatten(data: Dict, parent_key: Optional[str] = None) -> Dict:
    items = []
    for k, v in data.items():
        cur_key = f"{parent_key}.{k}" if parent_key is not None else k
        if isinstance(v, dict):
            items.extend(flatten(v, cur_key).items())
        else:
            items.append((cur_key, v))

    return dict(items)


def unflatten(data: Dict) -> Dict:
    out = dict()
    for k, v in data.items():
        parts = k.split(".")
        d = out
        for p in parts[:-1]:
            if p not in d:
                d[p] = dict()
            d = d[p]
        d[parts[-1]] = v
    return out



from __future__ import annotations
import os
import string
import sys
from typing import Optional


def modulepath(modulename: str) -> str:
    m = sys.modules[modulename]
    f = getattr(m, '__file__', None)
    if not f:
        return os.path.abspath(os.getcwd())
    return os.path.dirname(os.path.abspath(f))


def splitpath(path: str, sep: Optional[str] = None) -> list:
    """ Split the path into a list of items """
    return list(filter(len, path.split(sep or os.path.sep)))


def strip_path_inclusion(path: str, base: str) -> str:
    path = path.replace("\\", os.path.sep).replace("/", os.path.sep)
    return path if os.path.abspath(os.path.join(base, path)).startswith(base) else os.path.basename(path)


ASCII_NUMBERS = list(range(48, 58))
ASCII_UPPER_CASE_LETTERS = list(range(65, 91))
ASCII_LOWER_CASE_LETTERS = list(range(97, 123))
ASCII_OTHER = [45, 46, 92]  # "-", ".", and "\"

# Create a set that contains all of the valid characters that
# are allowed to appear in a Unified Naming Convention (UNC) path.
VALID_UNC_CHARS = [chr(x) for x in ASCII_LOWER_CASE_LETTERS +
                   ASCII_UPPER_CASE_LETTERS + ASCII_NUMBERS + ASCII_OTHER]


def is_unc_legal(path: str) -> bool:
    """Determine whether or not a given string representing a Windows file path is legal
    or not as per the Unified Naming Convention (UNC) specifications."""
    if len(path) <= 0:
        return False

    for char in path:
        if char not in VALID_UNC_CHARS:
            return False
    return True



import logging
import os
import posixpath
import tempfile
import warnings


# Stop Blowfish deprecation warning
with warnings.catch_warnings():
    warnings.simplefilter("ignore")

    import pysftp

from io import BytesIO
from paramiko import SSHException

from assemblyline.common.exceptions import ChainAll
from assemblyline.common.uid import get_random_id
from assemblyline.filestore.transport.base import Transport, TransportException, normalize_srl_path


def reconnect_retry_on_fail(func):
    def new_func(self, *args, **kwargs):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")

            try:
                if not self.sftp:
                    self.sftp = pysftp.Connection(self.host,
                                                  username=self.user,
                                                  password=self.password,
                                                  private_key=self.private_key,
                                                  private_key_pass=self.private_key_pass,
                                                  port=self.port,
                                                  cnopts=self.cnopts)
                return func(self, *args, **kwargs)
            except SSHException:
                pass

            # The previous attempt at calling original func failed.
            # Reset the connection and try again (one time).
            if self.sftp:
                self.sftp.close()   # Just best effort.

            # The original func will reconnect automatically.
            self.sftp = pysftp.Connection(self.host,
                                          username=self.user,
                                          password=self.password,
                                          private_key=self.private_key,
                                          private_key_pass=self.private_key_pass,
                                          cnopts=self.cnopts)
            return func(self, *args, **kwargs)

    new_func.__name__ = func.__name__
    new_func.__doc__ = func.__doc__
    return new_func


@ChainAll(TransportException)
class TransportSFTP(Transport):
    """
    SFTP Transport class.
    """

    def __init__(self, base=None, host=None, password=None, user=None, port=None, private_key=None,
                 private_key_pass=None, validate_host=False):
        self.log = logging.getLogger('assemblyline.transport.sftp')
        if base == "/":
            self.base = "./"
        else:
            self.base = base
        self.host = host
        self.port = int(port or 22)
        self.password = password
        self.user = user
        self.private_key = private_key
        self.private_key_pass = private_key_pass
        if not validate_host:
            self.cnopts = pysftp.CnOpts()
            self.cnopts.hostkeys = None
        else:
            self.cnopts = None

        # Connect on create
        self.sftp = pysftp.Connection(self.host,
                                      username=self.user,
                                      password=self.password,
                                      private_key=self.private_key,
                                      private_key_pass=self.private_key_pass,
                                      port=self.port,
                                      cnopts=self.cnopts)

        def sftp_normalize(path):
            # If they've provided an absolute path. Leave it a is.
            if path.startswith('/'):
                s = path
            # Relative paths
            elif '/' in path or len(path) != 64:
                s = posixpath.join(self.base, path)
            else:
                s = posixpath.join(self.base, normalize_srl_path(path))
            self.log.debug('sftp normalized: %s -> %s', path, s)
            return s

        super(TransportSFTP, self).__init__(normalize=sftp_normalize)

    def __str__(self):
        return 'sftp://{}@{}{}'.format(self.user, self.host, self.base)

    def close(self):
        if self.sftp:
            self.sftp.close()

    @reconnect_retry_on_fail
    def delete(self, path):
        path = self.normalize(path)
        self.sftp.remove(path)

    @reconnect_retry_on_fail
    def exists(self, path):
        path = self.normalize(path)
        return self.sftp.exists(path)

    @reconnect_retry_on_fail
    def makedirs(self, path):
        path = self.normalize(path)
        self.sftp.makedirs(path)

    # File based functions
    @reconnect_retry_on_fail
    def download(self, src_path, dst_path):
        dir_path = os.path.dirname(dst_path)

        if not os.path.exists(dir_path):
            os.makedirs(dir_path)

        src_path = self.normalize(src_path)
        self.sftp.get(src_path, dst_path)

    @reconnect_retry_on_fail
    def upload(self, src_path, dst_path):
        dst_path = self.normalize(dst_path)
        dirname = posixpath.dirname(dst_path)
        filename = posixpath.basename(dst_path)
        tempname = get_random_id()
        temppath = posixpath.join(dirname, tempname)
        finalpath = posixpath.join(dirname, filename)
        assert (finalpath == dst_path)
        self.makedirs(dirname)
        self.sftp.put(src_path, temppath)
        self.sftp.rename(temppath, finalpath)
        assert (self.exists(dst_path))

    # Buffer based functions
    @reconnect_retry_on_fail
    def get(self, path: str) -> bytes:
        path = self.normalize(path)
        bio = BytesIO()
        with self.sftp.open(path) as sftp_handle:
            bio.write(sftp_handle.read())
        return bio.getvalue()

    @reconnect_retry_on_fail
    def put(self, dst_path, content):
        dst_path = self.normalize(dst_path)
        dirname = posixpath.dirname(dst_path)
        filename = posixpath.basename(dst_path)
        tempname = get_random_id()
        temppath = posixpath.join(dirname, tempname)
        finalpath = posixpath.join(dirname, filename)
        assert (finalpath == dst_path)

        # Validate content
        if isinstance(content, str):
            content_data = content.encode('utf-8')
        else:
            content_data = content

        # Write content to a tempfile
        fd, src_path = tempfile.mkstemp(prefix="filestore.local_path")
        with open(fd, "wb") as f:
            f.write(content_data)

        # Upload the tempfile
        self.makedirs(dirname)
        self.sftp.put(src_path, temppath)
        self.sftp.rename(temppath, finalpath)
        assert (self.exists(dst_path))

        # Cleanup
        os.unlink(src_path)



"""
Try to find the version of the code base that is running or assume an arbitrary one for testing.
"""
import os
import re


ASSEMBLYLINE_VERSION = os.environ.get('ASSEMBLYLINE_VERSION', "4.0.0.dev0")

PATTERN = r'v?(?P<framework>[0-9]+)\.(?P<system>[0-9]+)\.(?P<minor>[0-9]+)\.(?P<channel>dev|stable)?(?P<build>[0-9]+)'

matching = re.match(PATTERN, ASSEMBLYLINE_VERSION, re.IGNORECASE)
if matching:
    groups = matching.groupdict()
else:
    raise EnvironmentError("Could not process the ASSEMBLYLINE_VERSION variable to extract a release version.")

FRAMEWORK_VERSION = int(groups['framework'])
SYSTEM_VERSION = int(groups['system'])
BUILD_MINOR = int(groups['minor'])



from __future__ import annotations

import hashlib

from assemblyline import odm
from assemblyline.common import constants, forge

Classification = forge.get_classification()

SUBMISSION_STATES = ['failed', 'submitted', 'completed']
DEFAULT_SRV_SEL = ["Filtering", "Antivirus", "Static Analysis", "Extraction", "Networking"]
DEFAULT_RESUBMIT = []


@odm.model(index=True, store=False, description="File Model of Submission.")
class File(odm.Model):
    name = odm.Keyword(copyto="__text__", description="Name of the submission.")
    size = odm.Optional(odm.long(), description="Size of the submitted file in bytes.")
    sha256 = odm.SHA256(copyto="__text__", description="SHA256 hash of the submitted file.")


@odm.model(index=False, store=False, description="Service Selection Scheme.")
class ServiceSelection(odm.Model):
    selected = odm.List(odm.Keyword(), default=DEFAULT_SRV_SEL, description="List of selected services.")
    excluded = odm.List(odm.Keyword(), default=[], description="List of excluded services.")
    rescan = odm.List(
        odm.Keyword(),
        default=[],
        description="List of services to rescan when initial run scores as malicious.")
    resubmit = odm.List(odm.Keyword(), default=DEFAULT_RESUBMIT,
                        description="Add to service selection when resubmitting.")

# Fields in the parameters used to calculate hashes used for result caching
_KEY_HASHED_FIELDS = {
    'classification',
    'deep_scan',
    'ignore_cache',
    'ignore_recursion_prevention',
    'ignore_filtering',
    'ignore_size',
    'max_extracted',
    'max_supplementary',
}


@odm.model(index=True, store=False, description="Submission Parameters")
class SubmissionParams(odm.Model):
    classification = odm.Classification(default=Classification.UNRESTRICTED,
                                        description="Original classification of the submission.")
    deep_scan = odm.Boolean(default=False, description="Select to perform a deep scan.")
    description = odm.Text(store=True, copyto="__text__", description="User-supplied information applied to Submission Details.")
    generate_alert = odm.Boolean(default=False, description="Generate alert upon completion of analysis.")
    groups = odm.List(odm.Keyword(), default=[], description="List relevant group or organization related to this scan.")
    ignore_cache = odm.Boolean(default=False, description="Ignore cached service results.")
    ignore_recursion_prevention = odm.Boolean(default=False, description="Ignore recursions prevention to avoid performance issues.")
    ignore_filtering = odm.Boolean(default=False, description="Ignore services in the FILTER category (i.e. Safelist).")
    ignore_size = odm.Boolean(default=False, description="Ignore the file size limits.")
    never_drop = odm.Boolean(default=False, description="Ingestion of submission will not be dropped as a result of ingestion queue volume.")
    malicious = odm.Boolean(default=False, description="User confirmation that the submission is known to be malicious.")
    max_extracted = odm.Integer(default=500, description="Max number of extracted files.")
    max_supplementary = odm.Integer(default=500, description="Max number of supplementary files.")
    priority = odm.Integer(default=1000, description="Determines order in which submission is analyzed relative to the queue.", min=1, max=constants.MAX_PRIORITY)
    psid = odm.Optional(odm.UUID(), description="Submission ID of 'parent' submission that has not been resubmitted for extended scan.")
    quota_item = odm.Boolean(default=False, description="Does this submission count against quota?")
    services = odm.Compound(ServiceSelection, default={}, description="Identify which services will run in the relevant submission.")
    service_spec = odm.Mapping(odm.Mapping(odm.Any()), default={}, index=False, store=False,
                               description="Service-specific parameters for the relevant submission.")
    submitter = odm.Keyword(store=True, copyto="__text__", description="User who submitted the file.")
    trace = odm.boolean(default=False, description="Collect debug information about the processing of a submission.")
    ttl = odm.Integer(default=0, description="Time, in days, to live for this submission.")
    type = odm.Keyword(default="USER", description="Source of submission (i.e. 'USER' or a particular sensor).")
    initial_data = odm.Optional(odm.Text(index=False), description="Initialization for temporary submission data.")
    auto_archive = odm.Boolean(default=False,
                               description="Send submission to the archive upon completion of analysis.")
    delete_after_archive = odm.Boolean(
        default=False,
        description="When the submission is archived, immediately delete from hot storage.")
    use_archive_alternate_dtl = odm.Boolean(default=False,
                                            description="use alternating dtl when archiving.")

    def get_hashing_keys(self):
        """Get the sections of the submission parameters that should be used in result hashes."""
        data = self.as_primitives()
        return {k: v for k, v in data.items() if k in _KEY_HASHED_FIELDS}

    def create_filescore_key(self, sha256, services: dict = None):
        """This is the key used to store the final score of a submission for fast lookup.

        This lookup is one of the methods used to check for duplication in ingestion process,
        so this key is fairly sensitive.
        """
        # TODO do we need this version thing still be here?
        # One up this if the cache is ever messed up and we
        # need to quickly invalidate all old cache entries.
        version = 0

        if services is None:
            services = self.services.as_primitives()

        data = self.get_hashing_keys()
        data['service_spec'] = sorted((key, sorted(values.items())) for key, values in self.service_spec.items())
        data['sha256'] = sha256
        data['services'] = sorted((key, sorted(values)) for key, values in services.items())

        s = ', '.join([f"{k}: {data[k]}" for k in sorted(data.keys())])

        return 'v'.join([str(hashlib.md5(s.encode()).hexdigest()), str(version)])


@odm.model(index=True, store=True, description="Submission-Relevant Times.")
class Times(odm.Model):
    completed = odm.Optional(odm.Date(store=False), description="Date at which the submission finished scanning.")
    submitted = odm.Date(default="NOW", description="Date at which the submission started scanning.")


@odm.model(index=True, store=False, description="Submission Verdict")
class Verdict(odm.Model):
    malicious = odm.List(odm.Keyword(), default=[], description="List all submissions that were labelled malicious by a specific user.")
    non_malicious = odm.List(
        odm.Keyword(),
        default=[],
        description="List all submissions that were labelled non-malicious by a specific user.")


@odm.model(index=False, store=False, description="A logging event describing the processing of a submission.")
class TraceEvent(odm.Model):
    timestamp = odm.Date(default="NOW")
    event_type = odm.keyword()
    service = odm.optional(odm.keyword())
    file = odm.optional(odm.SHA256())
    message = odm.optional(odm.keyword())


@odm.model(index=True, store=True, description="Model of Submission")
class Submission(odm.Model):
    archive_ts = odm.Optional(odm.Date(description="Timestamp at which the submission was archived.", ai=False))
    archived = odm.Boolean(default=False, description="Submission is present in the malware archive.", ai=False)
    classification = odm.Classification(description="Overall security classification of the submission.")
    tracing_events = odm.sequence(odm.compound(TraceEvent), default=[], index=False, store=False)
    error_count = odm.Integer(description="Total number of errors in the submission.", ai=False)
    errors: list[str] = odm.List(odm.Keyword(), store=False, description="List of error keys present in the submission.", ai=False)
    expiry_ts = odm.Optional(odm.Date(store=False), description="Timestamp for when the submission record expires.", ai=False)
    file_count = odm.Integer(description="Total number of files in the submission.", ai=False)
    files: list[File] = odm.List(odm.Compound(File), description="List of files that were originally submitted.")
    max_score = odm.Integer(description="The highest score across all files within a submission.")
    metadata: dict[str, str] = odm.FlatMapping(odm.MetadataValue(), default={}, store=False, copyto="__text__", description="Metadata associated with the submission.")
    params: SubmissionParams = odm.Compound(SubmissionParams, description="Submission parameter details.", ai=False)
    results: list[str] = odm.List(odm.wildcard(), store=False, description="List of result keys from the submission.", ai=False)
    sid: str = odm.UUID(copyto="__text__", description="The ID associated with a submission.")
    state = odm.Enum(values=SUBMISSION_STATES, description="State of the submission (ie. completed).", ai=False)
    to_be_deleted = odm.Boolean(
        default=False, description="This submission is going to be deleted as soon as it finishes.", ai=False)
    times = odm.Compound(Times, default={}, description="Submission-specific times.")
    verdict = odm.Compound(Verdict, default={}, description="Relates to the verdict of the submission (i.e. Malicious or Non-Malicious).", ai=False)
    from_archive = odm.Boolean(index=False, default=False, description="Was loaded from the archive.", ai=False)

    # the filescore key, used in deduplication. This is a non-unique key, that is
    # shared by submissions that may be processed as duplicates.
    scan_key = odm.Optional(odm.Keyword(store=False, index=False, ai=False))

    def is_submit(self):
        return self.state == 'submitted'

    def is_complete(self):
        return self.state == 'completed'

    def is_initial(self):
        return self.is_submit() and not self.params.psid



from assemblyline import odm
from assemblyline.common import forge
from assemblyline.odm.models.config import SubmissionProfileParams

Classification = forge.get_classification()

ENCODINGS = {"cart", "raw", "zip"}
VIEWS = {"report", "details"}


@odm.model(index=False, store=False, description="Model of User Settings")
class UserSettings(odm.Model):
    download_encoding = odm.Enum(values=ENCODINGS, default="cart",
                                 description="Default download encoding when downloading files")
    default_external_sources = odm.List(odm.Keyword(), default=[],
                                        description="List of sha256 sources to check by default")
    default_zip_password = odm.Text(
        default="infected",
        description="Default user-defined password for creating password protected ZIPs when downloading files"
    )
    default_metadata = odm.Mapping(odm.Text(), default={}, description="Default metadata to add to submissions")
    executive_summary = odm.Boolean(default=True, description="Should executive summary sections be shown?")
    expand_min_score = odm.Integer(default=500, description="Auto-expand section when score bigger then this")
    preferred_submission_profile = odm.Optional(odm.Text(), description="Preferred submission profile")
    submission_profiles = odm.Mapping(odm.Compound(SubmissionProfileParams), default={},
                                      description="Default submission profile settings")
    submission_view = odm.Enum(values=VIEWS, default="report", description="Default view for completed submissions")


DEFAULT_USER_PROFILE_SETTINGS = {
    "download_encoding": "cart",
    "default_external_sources": [],
    "default_zip_password": "infected",
    "executive_summary": True,
    "expand_min_score": 500,
    "submission_view": "report",
    "default_metadata": {}
}

DEFAULT_SUBMISSION_PROFILE_SETTINGS = {
    "classification": Classification.UNRESTRICTED,
    "deep_scan": False,
    "generate_alert": False,
    "ignore_cache": False,
    "ignore_recursion_prevention": False,
    "ignore_filtering": False,
    "priority": 1000,
    "service_spec": {},
    "services": {},
    "ttl": 30
}



from assemblyline import odm
from assemblyline.odm.models.ontology.filetypes import PE


@odm.model(description="File Characteristics")
class File(odm.Model):
    # Common information
    md5 = odm.MD5(description="MD5 of file")
    sha1 = odm.SHA1(description="SHA1 of file")
    sha256 = odm.SHA256(description="SHA256 of file")
    type = odm.Optional(odm.Keyword(description="Type of file as identified by Assemblyline"))
    size = odm.Integer(description="Size of the file in bytes")
    names = odm.Optional(odm.List(odm.Text()), description="Known filenames associated to file")
    parent = odm.Optional(odm.SHA256(), description="Absolute parent of file relative to submission")

    # Specialized information (List from Tagging.File)
    # apk = odm.Optional(odm.Compound(APK), description="APK File Properties")
    # jar = odm.Optional(odm.Compound(JAR), description="JAR File Properties")
    # img = odm.Optional(odm.Compound(IMG), description="Image File Properties")
    # ole = odm.Optional(odm.Compound(OLE), description="OLE File Properties")
    pe = odm.Optional(odm.Compound(PE), description="Properties related to PE")
    # pdf = odm.Optional(odm.Compound(PDF), description="PDF File Properties")
    # plist = odm.Optional(odm.Compound(PList), description="PList File Properties")
    # powershell = odm.Optional(odm.Compound(PowerShell), description="PowerShell File Properties")
    # shortcut = odm.Optional(odm.Compound(Shortcut), description="Shortcut File Properties")
    # swf = odm.Optional(odm.Compound(SWF), description="SWF File Properties")



from assemblyline import odm


@odm.model(
    index=True,
    store=False,
    description="Top-level model containing all tagging metadata for an analysis.",
)
class Tagging(odm.Model):
    @odm.model(
        index=True,
        store=False,
        description="Attribution-related tags such as actors, campaigns, and families.",
    )
    class Attribution(odm.Model):
        actor = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Threat actors or groups attributed to this sample.",
        )
        campaign = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Named campaigns or operations associated with this sample.",
        )
        category = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="High-level attribution categories (e.g. crimeware, nation-state).",
        )
        exploit = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Named exploits or vulnerability identifiers used by this sample.",
        )
        implant = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Malware implants or tools linked to the attributed actor.",
        )
        family = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Malware families or codebases related to this sample.",
        )
        network = odm.Optional(
            odm.List(odm.UpperKeyword(copyto="__text__")),
            description="Network infrastructure or clusters used for attribution.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags derived from antivirus detections and heuristics.",
    )
    class AV(odm.Model):
        heuristic = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Antivirus heuristic names or identifiers triggered by the sample.",
        )
        virus_name = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Virus or malware names reported by antivirus engines.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Metadata tags extracted from digital certificates.",
    )
    class Cert(odm.Model):
        @odm.model(
            index=True,
            store=False,
            description="Certificate validity period (notBefore / notAfter).",
        )
        class CertValid(odm.Model):
            start = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Earliest date from which the certificate is valid.",
            )
            end = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Latest date until which the certificate is valid.",
            )

        extended_key_usage = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Extended key usage values indicating allowed certificate purposes.",
        )
        issuer = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Issuer distinguished name fields for the certificate.",
        )
        key_usage = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Key usage flags describing how the certificate key may be used.",
        )
        owner = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Subject entity that owns or controls the certificate.",
        )
        serial_no = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Certificate serial numbers.",
        )
        signature_algo = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Signature algorithm used to sign the certificate.",
        )
        subject = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Certificate subject distinguished name.",
        )
        subject_alt_name = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Subject alternative names (e.g. DNS names, IPs, emails).",
        )
        thumbprint = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Certificate thumbprints (hashes of the full certificate).",
        )
        valid = odm.Optional(
            odm.Compound(CertValid),
            description="Structured validity period information for the certificate.",
        )
        version = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Certificate version numbers.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags describing code-level relationships between samples.",
    )
    class Code(odm.Model):
        sha256 = odm.Optional(
            odm.List(odm.SHA256(copyto="__text__")),
            description="SHA256 hashes of related code blobs, modules, or snippets.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags produced by dynamic/sandbox analysis about runtime behavior.",
    )
    class Dynamic(odm.Model):
        @odm.model(
            index=True,
            store=False,
            description="Processes observed during dynamic execution.",
        )
        class DynamicProcess(odm.Model):
            command_line = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Command-line strings for processes started at runtime.",
            )
            file_name = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Executable or script filenames launched by the sample.",
            )
            shortcut = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Shortcut (.lnk) names or targets created or accessed.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Dynamic analysis signatures that fired.",
        )
        class DynamicSignature(odm.Model):
            category = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="High-level behavioral category for the dynamic signature.",
            )
            family = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Malware family name associated with the dynamic signature.",
            )
            name = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Human-readable name of the dynamic analysis signature.",
            )

        @odm.model(
            index=True,
            store=False,
            description="SSDeep-based similarity hashes for dynamic artifacts.",
        )
        class DynamicSSDeep(odm.Model):
            cls_ids = odm.Optional(
                odm.List(odm.SSDeepHash(copyto="__text__")),
                description="SSDeep hashes of CLSID-like identifiers seen during analysis.",
            )
            dynamic_classes = odm.Optional(
                odm.List(odm.SSDeepHash(copyto="__text__")),
                description="SSDeep hashes of dynamically loaded classes or COM objects.",
            )
            regkeys = odm.Optional(
                odm.List(odm.SSDeepHash(copyto="__text__")),
                description="SSDeep hashes of registry key strings accessed at runtime.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Raw Windows-related identifiers from dynamic analysis.",
        )
        class DynamicWindow(odm.Model):
            cls_ids = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="CLSIDs or similar identifiers observed during execution.",
            )
            dynamic_classes = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Names of dynamically loaded classes or COM objects.",
            )
            regkeys = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Registry key paths accessed or modified.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Operating system environment in the sandbox.",
        )
        class DynamicOperatingSystem(odm.Model):
            platform = odm.Optional(
                odm.List(odm.Platform(copyto="__text__")),
                description="OS platform identifiers (e.g. Windows, Linux).",
            )
            version = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="OS version strings observed (e.g. 10.0.19045).",
            )
            processor = odm.Optional(
                odm.List(odm.Processor(copyto="__text__")),
                description="CPU architecture (e.g. x86, x64) used in the sandbox.",
            )

        autorun_location = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Locations where persistence or autorun entries were created.",
        )
        dos_device = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="DOS device paths (e.g. \\\\.\\) referenced during execution.",
        )
        mutex = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Mutex names used for synchronization or infection markers.",
        )
        registry_key = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Registry keys created, read, or modified at runtime.",
        )
        process = odm.Optional(
            odm.Compound(DynamicProcess),
            description="Structured process information from sandbox execution.",
        )
        signature = odm.Optional(
            odm.Compound(DynamicSignature),
            description="Structured list of sandbox or dynamic signatures that fired.",
        )
        ssdeep = odm.Optional(
            odm.Compound(DynamicSSDeep),
            description="SSDeep-based fingerprints derived from dynamic artifacts.",
        )
        window = odm.Optional(
            odm.Compound(DynamicWindow),
            description="Windows opened during dynamic analysis.",
        )
        operating_system = odm.Optional(
            odm.Compound(DynamicOperatingSystem),
            description="Operating-system metadata from the sandbox environment.",
        )
        processtree_id = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Identifiers for nodes in the sandbox process tree.",
        )

    @odm.model(
        index=True,
        store=False,
        description="General informational tags extracted from content.",
    )
    class Info(odm.Model):
        phone_number = odm.Optional(
            odm.List(odm.PhoneNumber(copyto="__text__")),
            description="Phone numbers extracted from the sample.",
        )
        password = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Passwords or password-like strings extracted from the sample.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags describing file structure, content, and embedded formats.",
    )
    class File(odm.Model):
        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from Android APK packages.",
        )
        class FileAPK(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="High-level information about the Android application.",
            )
            class FileAPKApp(odm.Model):
                label = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="User-facing application label shown on the device.",
                )
                version = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Application version strings from the manifest.",
                )

            @odm.model(
                index=True, store=False, description="Android SDK version requirements."
            )
            class FileAPKSDK(odm.Model):
                min = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Minimum Android SDK/API level required to run the app.",
                )
                target = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Target Android SDK/API level the app was built for.",
                )

            activity = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Declared Android activities within the APK.",
            )
            app = odm.Optional(
                odm.Compound(FileAPKApp),
                description="Application-level information from the APK.",
            )
            feature = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Optional hardware or software features requested by the app.",
            )
            locale = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Locales or languages supported by the application.",
            )
            permission = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Android permissions requested by the application.",
            )
            pkg_name = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Application package names (e.g. com.example.app).",
            )
            provides_component = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Components exposed by the APK (activities, services, providers, etc.).",
            )
            sdk = odm.Optional(
                odm.Compound(FileAPKSDK),
                description="Structured Android SDK version information for the app.",
            )
            used_library = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Third-party or system libraries referenced by the APK.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Timestamp-related metadata associated with the file.",
        )
        class FileDate(odm.Model):
            creation = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="File creation timestamps.",
            )
            last_modified = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="File last-modified timestamps.",
            )

        @odm.model(
            index=True, store=False, description="Metadata extracted from ELF binaries."
        )
        class FileELF(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Information about individual ELF sections.",
            )
            class FileELFSections(odm.Model):
                name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Names of sections within the ELF file.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Information about ELF program segments.",
            )
            class FileELFSegments(odm.Model):
                type = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Segment type identifiers (e.g. LOAD, DYNAMIC).",
                )

            @odm.model(
                index=True,
                store=False,
                description="Metadata contained in ELF NOTE segments.",
            )
            class FileELFNotes(odm.Model):
                name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="ELF note or owner names.",
                )
                type = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="ELF note type identifiers.",
                )
                type_core = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Core-dump related ELF note type identifiers.",
                )

            libraries = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Shared libraries linked by the ELF file.",
            )
            interpreter = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Dynamic loader or interpreter path used by the ELF.",
            )
            sections = odm.Optional(
                odm.Compound(FileELFSections),
                description="Structured metadata for ELF sections.",
            )
            segments = odm.Optional(
                odm.Compound(FileELFSegments),
                description="Structured metadata for ELF program segments.",
            )
            notes = odm.Optional(
                odm.Compound(FileELFNotes),
                description="Structured metadata for ELF notes.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from image files and containers.",
        )
        class FileIMG(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Exiftool-derived metadata about the image.",
            )
            class FileIMGExiftool(odm.Model):
                creator_tool = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Application or tool reported as having created the image.",
                )
                derived_document_id = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Identifier for a document derived from the original source.",
                )
                document_id = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Original document identifier stored in metadata.",
                )
                instance_id = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Unique identifier for this specific file instance.",
                )
                toolkit = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Toolkit or library used to generate or edit the image.",
                )

            exif_tool = odm.Optional(
                odm.Compound(FileIMGExiftool),
                description="Exiftool metadata for the image.",
            )
            mega_pixels = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Image size expressed in megapixels.",
            )
            mode = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Mode field from image metadata, typically indicating how the image was captured or encoded.",
            )
            size = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Image dimensions or overall size information.",
            )
            sorted_metadata_hash = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Hash of normalized and sorted metadata fields.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from Java JAR archives.",
        )
        class FileJAR(odm.Model):
            main_class = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Main class specified in the JAR manifest.",
            )
            main_package = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Package containing the main class.",
            )
            imported_package = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Referenced or imported Java packages.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Observed file name variants and anomalies.",
        )
        class FileName(odm.Model):
            anomaly = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Suspicious or unusual filename patterns.",
            )
            extracted = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Names of files extracted from the original sample.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from OLE/Office compound documents.",
        )
        class FileOLE(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Information about embedded OLE macros.",
            )
            class FileOLEMacro(odm.Model):
                sha256 = odm.Optional(
                    odm.List(odm.SHA256(copyto="__text__")),
                    description="SHA256 hashes of extracted macro streams.",
                )
                suspicious_string = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Strings from macros that were flagged as suspicious.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Standard document summary properties from OLE.",
            )
            class FileOLESummary(odm.Model):
                author = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Document author metadata.",
                )
                codepage = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Character encoding or code page information.",
                )
                comment = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Document comments or summary notes.",
                )
                company = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Company or organization name from metadata.",
                )
                create_time = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Original document creation timestamp.",
                )
                last_printed = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Timestamp when the document was last printed.",
                )
                last_saved_by = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="User name that last saved the document.",
                )
                last_saved_time = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Timestamp when the document was last saved.",
                )
                manager = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Manager field from document properties.",
                )
                subject = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Document subject or brief description.",
                )
                title = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Document title string.",
                )

            macro = odm.Optional(
                odm.Compound(FileOLEMacro),
                description="Structured metadata describing macros embedded in the file.",
            )
            summary = odm.Optional(
                odm.Compound(FileOLESummary),
                description="Structured summary/document-property metadata.",
            )
            clsid = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Class IDs (CLSIDs) for embedded OLE objects.",
            )
            dde_link = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Dynamic Data Exchange (DDE) link targets.",
            )
            fib_timestamp = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Timestamps from the File Information Block (FIB).",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata and analysis artifacts from PDF documents.",
        )
        class FilePDF(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Date-related metadata fields from the PDF.",
            )
            class FilePDFDate(odm.Model):
                modified = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="PDF modification timestamps.",
                )
                pdfx = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="PDF/X standard-related metadata values.",
                )
                source_modified = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Timestamp when the source document was last modified.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Metadata about JavaScript embedded in the PDF.",
            )
            class FilePDFJavascript(odm.Model):
                sha1 = odm.Optional(
                    odm.List(odm.SHA1(copyto="__text__")),
                    description="SHA1 hashes of JavaScript streams found in the PDF.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Statistical fingerprints for the PDF structure.",
            )
            class FilePDFStats(odm.Model):
                sha1 = odm.Optional(
                    odm.List(odm.SHA1(copyto="__text__")),
                    description="SHA1 hashes representing PDF structural statistics.",
                )

            date = odm.Optional(
                odm.Compound(FilePDFDate),
                description="Structured collection of PDF date-related metadata.",
            )
            javascript = odm.Optional(
                odm.Compound(FilePDFJavascript),
                description="Structured metadata about JavaScript embedded inside the PDF.",
            )
            stats = odm.Optional(
                odm.Compound(FilePDFStats),
                description="Structured statistics metadata describing the PDF layout.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from Windows PE executables and libraries.",
        )
        class FilePE(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Debug directory information from the PE file.",
            )
            class FilePEDebug(odm.Model):
                guid = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Debug GUIDs (e.g. PDB signature identifiers).",
                )

            @odm.model(
                index=True,
                store=False,
                description="Information about exported PE functions.",
            )
            class FilePEExports(odm.Model):
                function_name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Names of functions exported by the PE file.",
                )
                module_name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Name of the module (DLL/EXE) providing the export.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Information and fingerprints of imported PE functions.",
            )
            class FilePEImports(odm.Model):
                fuzzy = odm.Optional(
                    odm.List(odm.SSDeepHash(copyto="__text__")),
                    description="SSDeep hashes computed over the import table.",
                )
                md5 = odm.Optional(
                    odm.List(odm.MD5(copyto="__text__")),
                    description="MD5 hashes representing imported symbols or modules.",
                )
                imphash = odm.Optional(
                    odm.List(odm.MD5(copyto="__text__")),
                    description="Canonical import-hash (imphash) values for the PE.",
                )
                sorted_fuzzy = odm.Optional(
                    odm.List(odm.SSDeepHash(copyto="__text__")),
                    description="Fuzzy hashes computed over sorted import entries.",
                )
                sorted_sha1 = odm.Optional(
                    odm.List(odm.SHA1(copyto="__text__")),
                    description="SHA1 hashes computed over sorted import entries.",
                )
                gimphash = odm.Optional(
                    odm.List(odm.SHA256(copyto="__text__")),
                    description="Go-style import-hash values for Go binaries.",
                )
                suspicious = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Flags or descriptors for suspicious import patterns.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Metadata related to the PE linker.",
            )
            class FilePELinker(odm.Model):
                timestamp = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Linker timestamp value from the PE header.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Metadata about the PE original entry point (OEP).",
            )
            class FilePEOEP(odm.Model):
                bytes = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Raw bytes taken around the entry point.",
                )
                hexdump = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Hexadecimal dump of bytes at the entry point.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Metadata about embedded PE resources.",
            )
            class FilePEResources(odm.Model):
                language = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Resource language identifiers.",
                )
                name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Resource names or identifiers.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Information about the PE Rich header.",
            )
            class FilePERichHeader(odm.Model):
                hash = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Hashes summarizing Rich header contents.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Information about sections within the PE file.",
            )
            class FilePESections(odm.Model):
                hash = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Hashes of section contents or characteristics.",
                )
                name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Section names (e.g. .text, .rsrc).",
                )

            @odm.model(
                index=True,
                store=False,
                description="Version-information resources from the PE file.",
            )
            class FilePEVersions(odm.Model):
                description = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Product or file description from version info.",
                )
                filename = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Original filename recorded in version info.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Authenticode signature and catalog metadata.",
            )
            class FilePEAuthenticode(odm.Model):
                @odm.model(
                    index=True,
                    store=False,
                    description="SpcSpOpusInfo attributes describing the signed program.",
                )
                class FilePEAuthenticodeSpcSpOpusInfo(odm.Model):
                    program_name = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Program name string from the Authenticode signature.",
                    )

                spc_sp_opus_info = odm.Optional(
                    odm.Compound(FilePEAuthenticodeSpcSpOpusInfo),
                    description="SpcSpOpusInfo metadata about the signed program.",
                )

            api_vector = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Vector of imported or used APIs summarizing sample behavior.",
            )
            authenticode = odm.Optional(
                odm.Compound(FilePEAuthenticode),
                description="Authenticode signature metadata for the PE.",
            )
            debug = odm.Optional(
                odm.Compound(FilePEDebug),
                description="Debug directory metadata from the PE.",
            )
            exports = odm.Optional(
                odm.Compound(FilePEExports), description="PE export table metadata."
            )
            imports = odm.Optional(
                odm.Compound(FilePEImports),
                description="PE import table metadata and associated hashes.",
            )
            linker = odm.Optional(
                odm.Compound(FilePELinker), description="PE linker metadata."
            )
            oep = odm.Optional(
                odm.Compound(FilePEOEP),
                description="Entry-point bytes and hexdump information.",
            )
            pdb_filename = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Names or paths of referenced PDB debug symbol files.",
            )
            resources = odm.Optional(
                odm.Compound(FilePEResources), description="PE resource metadata."
            )
            rich_header = odm.Optional(
                odm.Compound(FilePERichHeader),
                description="Rich header metadata for the PE.",
            )
            sections = odm.Optional(
                odm.Compound(FilePESections),
                description="Metadata describing PE sections.",
            )
            versions = odm.Optional(
                odm.Compound(FilePEVersions), description="Version resource metadata."
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from Apple property list (plist) files.",
        )
        class FilePList(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Build-environment metadata from the plist.",
            )
            class FilePListBuild(odm.Model):
                machine_os = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Operating system version of the build machine.",
                )

            @odm.model(
                index=True, store=False, description="CFBundle-related bundle metadata."
            )
            class FilePListCFBundle(odm.Model):
                @odm.model(
                    index=True,
                    store=False,
                    description="Bundle version metadata.",
                )
                class FilePListCFBundleVersion(odm.Model):
                    long = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Full or long-form bundle version string.",
                    )
                    short = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Short marketing version string.",
                    )

                development_region = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Default localization or development region.",
                )
                display_name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Human-readable application display name.",
                )
                executable = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Name of the main executable binary.",
                )
                identifier = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Bundle identifier string (e.g. com.example.app).",
                )
                name = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Internal bundle name.",
                )
                pkg_type = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Package type code (e.g. APPL).",
                )
                signature = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Legacy creator/signature code values.",
                )
                url_scheme = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Custom URL schemes registered by the application.",
                )
                version = odm.Optional(
                    odm.Compound(FilePListCFBundleVersion),
                    description="Structured bundle version information.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Developer tools (DT*) metadata fields.",
            )
            class FilePListDT(odm.Model):
                @odm.model(
                    index=True,
                    store=False,
                    description="Platform-specific build metadata.",
                )
                class FilePListDTPlatform(odm.Model):
                    build = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Platform build identifier.",
                    )
                    name = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Platform name (e.g. iPhoneOS, MacOSX).",
                    )
                    version = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Platform version number.",
                    )

                compiler = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Compiler or build tool identifier.",
                )
                platform = odm.Optional(
                    odm.Compound(FilePListDTPlatform),
                    description="Structured platform metadata used for building the app.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Launch Services (LS*) metadata from the plist.",
            )
            class FilePListLS(odm.Model):
                background_only = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Indicates whether the app is background-only.",
                )
                min_system_version = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Minimum operating system version required by Launch Services.",
                )

            @odm.model(
                index=True,
                store=False,
                description="Cocoa (NS*) behavior flags from the plist.",
            )
            class FilePListNS(odm.Model):
                apple_script_enabled = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Whether AppleScript automation is allowed for the app.",
                )
                principal_class = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Name of the app's principal Objective-C class.",
                )

            @odm.model(
                index=True,
                store=False,
                description="User-interface-related plist keys.",
            )
            class FilePListUI(odm.Model):
                background_modes = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="UI background modes the app declares.",
                )
                requires_persistent_wifi = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Indicates if the app requires persistent Wi-Fi connectivity.",
                )

            @odm.model(
                index=True, store=False, description="WatchKit or WK* related metadata."
            )
            class FilePListWK(odm.Model):
                app_bundle_identifier = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="Bundle identifier of the associated application.",
                )

            installer_url = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="URL used to obtain or install the app.",
            )
            min_os_version = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Minimum OS version required to run the software.",
            )
            requests_open_access = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Indicates whether a component (e.g. keyboard) requests full access.",
            )

            build = odm.Optional(
                odm.Compound(FilePListBuild),
                description="Structured build-environment details from the plist.",
            )
            cf_bundle = odm.Optional(
                odm.Compound(FilePListCFBundle),
                description="Structured CFBundle-related metadata.",
            )
            dt = odm.Optional(
                odm.Compound(FilePListDT),
                description="Structured developer tools (DT*) metadata.",
            )
            ls = odm.Optional(
                odm.Compound(FilePListLS),
                description="Structured Launch Services configuration from the plist.",
            )
            ns = odm.Optional(
                odm.Compound(FilePListNS),
                description="Structured Cocoa (NS*) configuration and behaviors.",
            )
            ui = odm.Optional(
                odm.Compound(FilePListUI),
                description="Structured UI behavior metadata.",
            )
            wk = odm.Optional(
                odm.Compound(FilePListWK),
                description="Structured WatchKit (WK*) metadata.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from PowerShell files or commands.",
        )
        class FilePowerShell(odm.Model):
            cmdlet = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="PowerShell cmdlets referenced or invoked by the script.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata from Windows shortcut (.lnk) files.",
        )
        class FileShortcut(odm.Model):
            command_line = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Command line stored in or invoked by the shortcut.",
            )
            icon_location = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Path of the icon referenced by the shortcut.",
            )
            machine_id = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Machine identifier recorded within the shortcut.",
            )
            tracker_mac = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Potential MAC addresses recovered from the shortcut tracker block.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Categorized strings extracted from the file.",
        )
        class FileStrings(odm.Model):
            api = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Extracted strings that resemble API or function names.",
            )
            blacklisted = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Strings matching blacklist patterns or known bad indicators.",
            )
            decoded = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Strings obtained after decoding or deobfuscation.",
            )
            extracted = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Raw printable strings extracted from the file.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata extracted from Adobe Flash (SWF) files.",
        )
        class FileSWF(odm.Model):
            @odm.model(
                index=True,
                store=False,
                description="Header-level metadata from the SWF file.",
            )
            class FileSWFHeader(odm.Model):
                @odm.model(
                    index=True,
                    store=False,
                    description="Frame-rate and size information from the SWF header.",
                )
                class FileSWFHeaderFrame(odm.Model):
                    count = odm.Optional(
                        odm.List(odm.Integer()),
                        description="Total number of frames in the SWF animation.",
                    )
                    rate = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Frame rate (speed) of the SWF animation.",
                    )
                    size = odm.Optional(
                        odm.List(odm.Keyword(copyto="__text__")),
                        description="Logical stage size or frame dimensions.",
                    )

                frame = odm.Optional(
                    odm.Compound(FileSWFHeaderFrame),
                    description="Structured SWF header frame information.",
                )
                version = odm.Optional(
                    odm.List(odm.Keyword(copyto="__text__")),
                    description="SWF file format version.",
                )

            header = odm.Optional(
                odm.Compound(FileSWFHeader),
                description="Structured SWF header metadata.",
            )
            tags_ssdeep = odm.Optional(
                odm.List(odm.SSDeepHash(copyto="__text__")),
                description="SSDeep hashes computed over SWF tags for similarity.",
            )

        ancestry = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Tags describing file genealogy or derivation relationships.",
        )
        behavior = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Behavioral characteristics inferred from analysis.",
        )
        compiler = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Compiler or toolchain used to build the file.",
        )
        config = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Configuration blocks or key-value settings extracted from the file.",
        )
        date = odm.Optional(
            odm.Compound(FileDate),
            description="Structured date and timestamp metadata for the file.",
        )
        elf = odm.Optional(
            odm.Compound(FileELF),
            description="Structured properties specific to ELF binaries.",
        )
        lib = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Libraries the file depends on or bundles.",
        )
        lsh = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Locality-sensitive hashes (LSH) computed for fuzzy similarity.",
        )
        name = odm.Optional(
            odm.Compound(FileName),
            description="Structured tags describing observed file names and anomalies.",
        )
        path = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="File system or archive paths where the file was seen.",
        )
        rule = odm.Optional(
            odm.Mapping(odm.List(odm.Keyword(copyto="__text__"))),
            description="Rules or signatures that matched this file, grouped by source.",
        )
        string = odm.Optional(
            odm.Compound(FileStrings),
            description="Structured categories of strings extracted from the file.",
        )
        apk = odm.Optional(
            odm.Compound(FileAPK),
            description="Detailed properties specific to Android APK files.",
        )
        jar = odm.Optional(
            odm.Compound(FileJAR),
            description="Detailed properties specific to Java JAR archives.",
        )
        img = odm.Optional(
            odm.Compound(FileIMG),
            description="Detailed properties specific to image files.",
        )
        ole = odm.Optional(
            odm.Compound(FileOLE),
            description="Detailed properties specific to OLE/Office documents.",
        )
        pe = odm.Optional(
            odm.Compound(FilePE),
            description="Detailed properties specific to Windows PE binaries.",
        )
        pdf = odm.Optional(
            odm.Compound(FilePDF),
            description="Detailed properties specific to PDF documents.",
        )
        plist = odm.Optional(
            odm.Compound(FilePList),
            description="Detailed properties specific to Apple plist files.",
        )
        powershell = odm.Optional(
            odm.Compound(FilePowerShell),
            description="Detailed properties specific to PowerShell scripts.",
        )
        shortcut = odm.Optional(
            odm.Compound(FileShortcut),
            description="Detailed properties specific to Windows shortcut files.",
        )
        swf = odm.Optional(
            odm.Compound(FileSWF),
            description="Detailed properties specific to SWF files.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags for network indicators and traffic-related artifacts.",
    )
    class Network(odm.Model):
        @odm.model(
            index=True,
            store=False,
            description="Network indicators of compromise (IOCs).",
        )
        class NetworkIOCs(odm.Model):
            domain = odm.Optional(
                odm.List(odm.Domain(copyto="__text__")),
                description="Domain names contacted, embedded, or otherwise referenced.",
            )
            ip = odm.Optional(
                odm.List(odm.IP(copyto="__text__")),
                description="IP addresses contacted, embedded, or otherwise referenced.",
            )
            unc_path = odm.Optional(
                odm.List(odm.UNCPath(copyto="__text__")),
                description="Windows UNC paths (\\\\server\\share) used by the sample.",
            )
            uri = odm.Optional(
                odm.List(odm.URI(copyto="__text__")),
                description="Full URIs or URLs observed (including scheme and host).",
            )
            uri_path = odm.Optional(
                odm.List(odm.URIPath(copyto="__text__")),
                description="URI path components without scheme or host.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Metadata from email-related network artifacts.",
        )
        class NetworkEmail(odm.Model):
            address = odm.Optional(
                odm.List(odm.Email(copyto="__text__")),
                description="Sender or recipient email addresses observed.",
            )
            date = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Email date header values.",
            )
            subject = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Email subject lines.",
            )
            msg_id = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Email Message-ID header values.",
            )

        @odm.model(
            index=True,
            store=False,
            description="Network IDS/IPS or rule-engine signatures.",
        )
        class NetworkSignature(odm.Model):
            signature_id = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Identifier of the network detection signature (e.g. SID).",
            )
            message = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Human-readable description of the network signature.",
            )

        @odm.model(
            index=True, store=False, description="TLS fingerprint and metadata tags."
        )
        class NetworkTLS(odm.Model):
            ja3_hash = odm.Optional(
                odm.List(odm.MD5(copyto="__text__")),
                description="MD5 hash of the JA3 TLS client fingerprint.",
            )
            ja3_string = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Raw JA3 TLS client fingerprint string.",
            )
            ja3s_hash = odm.Optional(
                odm.List(odm.MD5(copyto="__text__")),
                description="MD5 hash of the JA3S TLS server fingerprint.",
            )
            ja3s_string = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Raw JA3S TLS server fingerprint string.",
            )
            ja4_hash = odm.Optional(
                odm.List(
                    odm.ValidatedKeyword(
                        validation_regex=odm.JA4_REGEX, copyto="__text__"
                    )
                ),
                description="Validated JA4 TLS client fingerprint hash.",
            )
            ja4s_hash = odm.Optional(
                odm.List(
                    odm.ValidatedKeyword(
                        validation_regex=odm.JA4_REGEX, copyto="__text__"
                    )
                ),
                description="Validated JA4S TLS server fingerprint hash.",
            )
            sni = odm.Optional(
                odm.List(odm.Keyword(copyto="__text__")),
                description="Server Name Indication (SNI) values from TLS handshakes.",
            )

        attack = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="High-level classification of observed or attempted attacks.",
        )
        dynamic = odm.Optional(
            odm.Compound(NetworkIOCs),
            description="Network IOCs derived from dynamic/sandbox analysis.",
        )
        email = odm.Optional(
            odm.Compound(NetworkEmail),
            description="Structured email-related network metadata.",
        )
        mac_address = odm.Optional(
            odm.List(odm.MAC(copyto="__text__")),
            description="MAC addresses observed in network or related artifacts.",
        )
        port = odm.Optional(
            odm.List(odm.Integer()),
            description="Network port numbers used by the sample.",
        )
        protocol = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Application or transport protocols observed (e.g. HTTP, TCP).",
        )
        signature = odm.Optional(
            odm.Compound(NetworkSignature),
            description="Structured metadata for network detection signatures.",
        )
        static = odm.Optional(
            odm.Compound(NetworkIOCs),
            description="Network IOCs derived from static analysis of the sample.",
        )
        tls = odm.Optional(
            odm.Compound(NetworkTLS),
            description="Structured TLS handshake and fingerprint information.",
        )
        user_agent = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="HTTP or other user-agent strings observed.",
        )

    @odm.model(
        index=True,
        store=False,
        description="Tags capturing techniques and tradecraft used by the sample.",
    )
    class Technique(odm.Model):
        comms_routine = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Patterns or routines used for C2 or other communications.",
        )
        config = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Technique-related configuration data (e.g. keys, flags).",
        )
        crypto = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Use of cryptographic algorithms, keys, or primitives.",
        )
        exploit = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Exploit techniques or identifiers used by the sample.",
        )
        keylogger = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Keylogging components or behaviors.",
        )
        macro = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Macro-based execution techniques or mechanisms.",
        )
        masking_algo = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Algorithms used for masking, encoding, or hiding data.",
        )
        obfuscation = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Obfuscation or anti-analysis techniques observed.",
        )
        packer = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Packers or protectors used to wrap the sample.",
        )
        persistence = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Persistence techniques used to survive reboot or logoff.",
        )
        shellcode = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Shellcode payloads or shellcode-based techniques.",
        )
        string = odm.Optional(
            odm.List(odm.Keyword(copyto="__text__")),
            description="Technique-related string patterns (e.g. markers, protocol strings).",
        )

    attribution = odm.Optional(
        odm.Compound(Attribution),
        description="All attribution-related tags (actors, campaigns, tooling, etc.).",
    )
    av = odm.Optional(
        odm.Compound(AV),
        description="Tags derived from antivirus detection names and heuristics.",
    )
    cert = odm.Optional(
        odm.Compound(Cert),
        description="Tags derived from digital certificates and related fields.",
    )
    code = odm.Optional(
        odm.Compound(Code),
        description="Tags capturing relationships to other code samples.",
    )
    dynamic = odm.Optional(
        odm.Compound(Dynamic),
        description="Tags generated from sandbox or other dynamic analysis.",
    )
    info = odm.Optional(
        odm.Compound(Info),
        description="General informational tags not covered by other categories.",
    )
    file = odm.Optional(
        odm.Compound(File),
        description="Tags describing file content, structure, and embedded formats.",
    )
    network = odm.Optional(
        odm.Compound(Network),
        description="Tags describing network indicators and communication patterns.",
    )
    source = odm.Optional(
        odm.List(odm.Keyword(copyto="__text__")),
        description="Tags describing where the sample or tagging information originated.",
    )
    technique = odm.Optional(
        odm.Compound(Technique),
        description="Tags summarizing techniques, tactics, and tradecraft used.",
    )
    vector = odm.Optional(
        odm.List(odm.Keyword(copyto="__text__")),
        description="Tags describing delivery or infection vectors for the sample.",
    )

if __name__ == "__main__":
    from pprint import pprint

    pprint(list(Tagging().flat_fields().keys()))



from assemblyline import odm

MSG_TYPES = {"ScalerHeartbeat"}
LOADER_CLASS = "assemblyline.odm.messages.scaler_heartbeat.ScalerMessage"


@odm.model(description="Metrics")
class Metrics(odm.Model):
    memory_free = odm.Float(description="Amount of free memory")
    cpu_free = odm.Float(description="Amount of free CPU")
    memory_total = odm.Float(description="Amount of total memory")
    cpu_total = odm.Float(description="Amount of total CPU")


@odm.model(description="Heartbeat Model")
class Heartbeat(odm.Model):
    instances = odm.Integer(description="Number of instances")
    metrics = odm.Compound(Metrics, description="Metrics")


@odm.model(description="Model of Scaler Heartbeat Message")
class ScalerMessage(odm.Model):
    msg = odm.Compound(Heartbeat, description="Heartbeat message")
    msg_loader = odm.Enum(values={LOADER_CLASS}, default=LOADER_CLASS, description="Loader class of message")
    msg_type = odm.Enum(values=MSG_TYPES, default="ScalerHeartbeat", description="Type of message")
    sender = odm.Keyword(description="Sender of message")
